<?php

/**
 * Plugin Name: Pargo Shipping
 * Description: Pargo is a convenient logistics solution that lets you collect and return parcels at Pargo parcel points throughout the country when it suits you best.
 * Version: 2.5.9
 * Author: Pargo
 * Author URI: https://www.pargo.co.za
 * License: GPL-3.0+
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 */

// Global Variables
define('PARGOPATH', str_replace('\\', '/', rtrim(__DIR__, '/')) . '/');
define("PARGOPLUGINVERSION", "2.5.9");
define("PARGOPLUGINDIR", plugin_dir_path(__FILE__));
define("PARGOPLUGINURL", plugin_dir_url(__FILE__));
define("PARGOWCSETTINGS", 'admin.php?page=wc-settings&tab=shipping&section=wp_pargo');

require_once ABSPATH . '/wp-includes/pluggable.php';
require_once WP_PLUGIN_DIR . '/woocommerce/woocommerce.php';
include_once PARGOPATH . "includes/controllers/class-wp-pargo-shipping-method.php";
require_once PARGOPATH . "includes/controllers/class-wp-pargo-shipping-processes.php";
include_once PARGOPATH . "includes/controllers/class-wp-pargo-api.php";
include PARGOPATH . 'PargoOrders.php';
require plugin_dir_path(__FILE__) . 'includes/controllers/class-wp-pargo.php';

// Security check: prevent this plugin file being accessed from outside of WordPress.
// The constant ABSPATH or WPINC will not be defined, so the plugin code will exit.
if (!defined('WPINC')) {
    die('We\'re sorry, but you can not directly access this file.');
}
if (!defined('ABSPATH')) {
    die('We\'re sorry, but you can not directly access this file.');
}
WC()->session = new WC_Session_Handler();
WC()->session->init();

function set_defaults()
{
    // set default shipping method
    if (empty(WC()->session->get('delivery_type'))) {
        WC()->session->set('delivery_type', PargoOrders::PROCESS_TYPE_W2P);
    }
}
add_action('woocommerce_cart_totals_before_shipping', 'set_defaults');

// Hooks to the WooCommerce Thank You / Order Received function and posts the order to the Pargo API
add_action('woocommerce_order_status_changed', 'placePargoOrder');
add_action('woocommerce_new_order', 'placePargoOrder');
function placePargoOrder($order_id)
{
    // only post order if a waybill does not already exist
    if (get_post_meta($order_id, 'waybill', true)) {
        return;
    }

    $order = wc_get_order($order_id);
    if ('processing' === $order->get_status() && WC()->session->get('delivery_type')) {
        $pargoOrders = new PargoOrders();
        $pargoOrders->placeOrder($order, WC()->session->get('delivery_type'));
    }
}

// Hooks to the WooCommerce admin page, display link to MyPargo to view order details
add_action('woocommerce_admin_order_data_after_shipping_address', 'pargo_custom_row_after_order_addresses', 10, 1);
function pargo_custom_row_after_order_addresses($order)
{
    $meta = $order->get_meta_data();
    foreach ($meta as $key => $value) {
        if (in_array('pargo_pc', $value->get_data(), true)) {
            echo "
                </div>
                    </div>
                    <div class='clear'></div>
                    <!-- new custom section row -->
                    <div class='order_data_column_container'>
                    <div class='order_data_column_wide'>
                    <h3 style='margin-bottom: 5px'>'Pargo Shipment</h3>
                    <p>Login to <a href='https://mypargo.pargo.co.za'>MyPargo</a> to manage your order delivery.</p>";
        }
    }
}

function pargo_redirect()
{
    if (get_option('redirect')) {
        delete_option('redirect');
        wp_redirect('redirect');
        // wp_redirect() does not exit automatically and should almost always be followed by exit.
        exit;
    }
}

function pargo_plugin_redirect()
{
    if (get_option('redirect')) {
        wp_redirect(get_option('redirect'));
        delete_option('redirect');
        exit;
    }
}

// Check if WooCommerce is active
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {

    // Admin menu

    add_action('admin_menu', 'pargo_plugin_menu');
    function pargo_plugin_menu()
    {
        add_menu_page(
            "Pargo Plugin Options",
            "Pargo",
            "manage_options",
            "pargo-shipping",
            "pargo_accounts_page",
            "dashicons-cart"
        );

        add_submenu_page(
            "pargo-shipping", // Required. Slug of top level menu item
            "Pargo - Accounts", // Required. Text to be displayed in title.
            "Pargo Account", // Required. Text to be displayed in menu.
            "manage_options", // Required. The required capability of users.
            "pargo-shipping", // Required. A unique identifier to the sub menu item.
            "pargo_accounts_page", // Optional. This callback outputs the content of the page associated with this menu item.
            null // Optional. The URL of the menu item icon
        );

        add_submenu_page(
            "pargo-shipping", // Required. Slug of top level menu item
            "Pargo - Settings", // Required. Text to be displayed in title.
            "Pargo Settings", // Required. Text to be displayed in menu.
            "manage_options", // Required. The required capability of users.
            "pargo-shipping-method", // Required. A unique identifier to the sub menu item.
            "pargo_woocom_shipping", // Optional. This callback outputs the content of the page associated with this menu item.
            null // Optional. The URL of the menu item icon
        );

        add_submenu_page(
            "pargo-shipping", // Required. Slug of top level menu item
            "Pargo - Help", // Required. Text to be displayed in title.
            "Pargo Help", // Required. Text to be displayed in menu.
            "manage_options", // Required. The required capability of users.
            "pargo-help", // Required. A unique identifier to the sub menu item.
            "pargo_shipping_help_page", // Optional. This callback outputs the content of the page associated with this menu item.
            null // Optional. The URL of the menu item icon
        );
    }

    // Pargo options page
    function pargo_plugin_options()
    {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }
        include_once 'pargo-admin.php';
    }

    function pargo_woocom_shipping()
    {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }
        $settings_link = 'admin.php?page=wc-settings&tab=shipping&section=wp_pargo';
        header("Location: $settings_link");
    }

    // Pargo Support page
    function pargo_shipping_support_page()
    {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }

        include_once 'includes/views/admin/pargo-support.php';
    }

    // Pargo Support page
    function pargo_shipping_help_page()
    {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }

        include_once 'includes/views/admin/pargo-help.php';
    }

    // Pargo Accounts Page
    function pargo_accounts_page()
    {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }

        include_once 'includes/views/admin/pargo-credentials.php';
    }
    // EndOF-Admin Pages

    add_filter('woocommerce_checkout_get_value', 'set_shipping_zip', 10, 2);
    function set_shipping_zip()
    {
        $shippingProcesses = new Wp_Pargo_Shipping_Processes();
        $shippingProcesses->setShippingZip();
    }
    add_action('woocommerce_before_checkout_form', 'set_shipping_zip');

    function override_shipping_logic($fields)
    {
        $shippingProcesses = new Wp_Pargo_Shipping_Processes();
        return $shippingProcesses->overrideShippingLogic($fields);
    }
    add_filter('woocommerce_shipping_fields', 'override_shipping_logic');

    add_filter('woocommerce_default_address_fields', 'misha_remove_fields');
    function misha_remove_fields($fields)
    {
        $fields['suburb']   = [
            'label'        => 'Suburb',
            'required'     => true,
            'class'        => ['form-row-wide'],
            'priority'     => 20,
            'placeholder'  => '',
        ];
        return $fields;
    }

    function pargo_shipping_method()
    {
        if (!class_exists('Wp_Pargo_Shipping_Method')) {
            add_action('woocommerce_after_checkout_form', 'bbloomer_disable_shipping_local_pickup');

            function bbloomer_disable_shipping_local_pickup($available_gateways)
            {
                global $woocommerce;

                // Part 1: Hide shipping based on the static choice @ Cart
                // Note: "#customer_details .col-2" strictly depends on your theme
                $chosen_methods = WC()->session->get('chosen_shipping_methods');
                $chosen_shipping_no_ajax = $chosen_methods[0];
                if (0 === strpos($chosen_shipping_no_ajax, 'wp_pargo')) {
                    echo "
                    <script type='text/javascript'>
                        jQuery('#customer_details .col-2').fadeOut();
                    </script>
                ";
                }

                // Part 2: Hide shipping based on the dynamic choice @ Checkout
                // Note: "#customer_details .col-2" strictly depends on your theme
                echo "
                <script type='text/javascript'>
                    jQuery('form.checkout').on('change', 'input[name^=\"shipping_method\"]', function() {
                        var val = jQuery(this).val();
                        if (val.match(\"^wp_pargo\")) {
                            jQuery('#customer_details .col-2').fadeOut();
                        } else {
                            jQuery('#customer_details .col-2').fadeIn();
                        }
                    });
                </script>";
            }
        }
    }
    add_action('woocommerce_shipping_init', 'pargo_shipping_method');

    function add_pargo_shipping_method($methods)
    {
        $methods['wp_pargo'] = 'Wp_Pargo_Shipping_Method';
        return $methods;
    }
    add_filter('woocommerce_shipping_methods', 'add_pargo_shipping_method');

    // *Yellow Button and Display*
    add_filter('woocommerce_cart_shipping_method_full_label', 'wc_pargo_label_change', 10, 2);
    function wc_pargo_label_change($label, $method)
    {
        require_once PARGOPATH . "includes/controllers/class-wp-pargo-shipping-processes.php";
        $shippingProcesses = new Wp_Pargo_Shipping_Processes();
        return $shippingProcesses->wcPargoLabelChange($label, $method);
    }

    add_action('woocommerce_after_checkout_validation', 'pargo_after_checkout_validation');
    function pargo_after_checkout_validation($posted)
    {
        $chosen_methods = WC()->session->get('chosen_shipping_methods');
        $chosen_shipping = $chosen_methods[0];
        if ($chosen_shipping != 'wp_pargo') {
            return;
        }

        if (!empty(WC()->session->get('pargo_shipping_address'))) {
            add_action('woocommerce_checkout_update_order_meta', function ($order_id, $posted) {
                $post = WC()->session->get('pargo_shipping_address')['pargoPointCode'];
                $pargo_del_add = "" . WC()->session->get('pargo_shipping_address')['storeName'] . ", " . WC()->session->get('pargo_shipping_address')['storeName'] . ", " . WC()->session->get('pargo_shipping_address')['address1'] . ", " . WC()->session->get('pargo_shipping_address')['address2'] . ", " . WC()->session->get('pargo_shipping_address')['city'] . ", " . WC()->session->get('pargo_shipping_address')['province'] . ", " . WC()->session->get('pargo_shipping_address')['postalcode'] . "";
                $order = wc_get_order($order_id);
                $order->update_meta_data('pargo_pc', $post);
                $order->update_meta_data('pargo_delivery_add', $pargo_del_add);
                $order->save();
            }, 10, 2);
        }
    }

    /**
     * Display field value on the order edit page
     */
    add_action('woocommerce_admin_order_data_after_billing_address', 'pargo_checkout_field_display_admin_order_meta', 10, 1);
    function pargo_checkout_field_display_admin_order_meta($order)
    {
        $meta = $order->get_meta_data();

        foreach ($meta as $key => $value) {
            if (in_array('pargo_pc', $value->get_data())) {
                global $woocommerce;
                echo '<p><b>' . __('Pargo Pick Up Address') . ':</b> ' . get_post_meta($order->get_id(), 'pargo_delivery_add', true) . '</p>';
                echo '<p><b>' . __('Pargo Pick Up Point Code') . ':</b> ' . get_post_meta($order->get_id(), 'pargo_pc', true) . '</p>';
            }
        }
    }

    /**
     * Pargo Modals
     */
    add_action('wp_footer', 'pargo_modals');
    function pargo_modals()
    {
        echo '<div id="pargo-not-selected" role="dialog" aria-labelledby="mySmallModalLabel">' .
            '<div id="pargo_center">' .
            '<div id="pargo_inner">' .
            '<div class="pargo_blk_title"><img src="' . PARGOPLUGINURL . 'assets/images/alert.png" /></div>' .
            '<div class="pargo_close">x</div>' .
            '<div class="pargo_content">' .
            '<p>You forgot to select your pick up point!<br />' .
            'Remember, we have pick up locations throughout the country, just pick one!' .
            '</p><img src="' . PARGOPLUGINURL . 'assets/images/click_point.png" />' .
            '</div>' .
            '</div>' .
            '</div>' .
            '</div>';
    }

    /**
     * Validate the order
     */
    function pargo_validate_order($posted)
    {
        $shippingProcesses = new Wp_Pargo_Shipping_Processes();
        $shippingProcesses->pargoValidateOrders($posted);
    } //end function Pargo weight warning

    add_action('woocommerce_review_order_before_cart_contents', 'pargo_validate_order', 10);
    add_action('woocommerce_after_checkout_validation', 'pargo_validate_order', 10);

    add_action('woocommerce_order_details_after_order_table', 'pargo_custom_field_display_cust_order_meta', 10, 1);
    function pargo_custom_field_display_cust_order_meta($order)
    {
        // check if pargo is the chosen shipping method
        $chosen_methods = WC()->session->get('chosen_shipping_methods');
        $chosen_shipping = $chosen_methods[0];
        if ($chosen_shipping != 'wp_pargo') {
            return false;
        }

        if (isset(WC()->session->get('pargo_shipping_address')['storeName'])) {
            $storeName = WC()->session->get('pargo_shipping_address')['storeName'];
        }
        if (isset(WC()->session->get('pargo_shipping_address')['address1'])) {
            $storeAddress = WC()->session->get('pargo_shipping_address')['address1'];
        }
        if (
            isset(WC()->session->get('pargo_shipping_address')['address1'])
            && WC()->session->get('delivery_type') === Wp_Pargo_Shipping_Method::PARGO_PROCESS_TYPE_W2P
        ) {
            echo '<p><strong>' . __('Pargo Pickup Address') . ':</strong> ' . $storeName . '. ' . $storeAddress . '</p>';
        }
    }

    add_filter('style_loader_src', 'sdt_remove_ver_css_js', 9999, 2);
    add_filter('script_loader_src', 'sdt_remove_ver_css_js', 9999, 2);
    function sdt_remove_ver_css_js($src, $handle)
    {
        $handles_with_version = ['style']; // <-- Adjust to your needs!

        if (strpos($src, 'ver=') && !in_array($handle, $handles_with_version, true)) {
            $src = remove_query_arg('ver', $src);
        }

        return $src;
    }

    add_action('wp_footer', 'woocommerce_add_gift_box');
    function woocommerce_add_gift_box()
    {
        if (is_checkout()) {
            ?>
            <script type="text/javascript">
                jQuery(document).ready(function($) {
                    jQuery('#parcel_test').click(function() {
                        jQuery('body').trigger('update_checkout');
                    });
                });
            </script>
<?php
        }
    }

    if (get_option('woocommerce_wp_pargo_settings')['pargo_map_display'] === 'yes') {
        add_action('woocommerce_review_order_before_payment', function () {
            include_once PARGOPATH . 'includes/views/front_end/woocommerce_add_cart_fee.php';
        });
    }
}

add_filter('woocommerce_package_rates', 'show_shipping_method_on_order_weight', 10, 2);
function show_shipping_method_on_order_weight($available_shipping_methods, $package)
{
    $weight = WC()->cart->get_cart_contents_weight();
    if ($weight > 15) {
        unset($available_shipping_methods['wp_pargo']);
    }

    return $available_shipping_methods;
}

/**
 * The Opt-In / Opt-Out Handler
 */
add_action('wp_ajax_pargo_optin', 'pargo_optin');
add_action('wp_ajax_nopriv_pargo_optin', 'pargo_optin');
function pargo_optin()
{
    $pargo_shipping_method = new Wp_Pargo_Shipping_Method();
    foreach ($_POST as $key => $value) {
        $pargo_shipping_method->update_option($key, $value);
    }

    exit("Option Saved Successfully");
}

/**
 * The Credentials form handler.
 */
add_action('wp_ajax_pargo_account_credentials', 'pargo_account_credentials');
add_action('wp_ajax_nopriv_pargo_account_credentials', 'pargo_account_credentials');
function pargo_account_credentials()
{
    $pargo_shipping_method = new Wp_Pargo_Shipping_Method();

    foreach ($_POST as $key => $value) {
        $pargo_shipping_method->update_option($key, $value);
    }

    // Don't forget to exit at the end of processing
    exit("Credentials have been Saved Successfully.");
}

/**
 * The Verify Credentials form handler.
 */
add_action('wp_ajax_pargo_verify_credentials', 'pargo_verify_credentials');
add_action('wp_ajax_nopriv_pargo_verify_credentials', 'pargo_verify_credentials');
function pargo_verify_credentials()
{
    $pargoApi = new Wp_Pargo_Api();
    $result = $pargoApi->verifyUserCredentials($_POST['api_url'], $_POST['username'], $_POST['password']);
    if ($result) {
        exit("Account verified successfully.");
    } else {
        exit("Account could not be verified");
    }
}

add_action('wp_ajax_set_delivery_type', 'set_delivery_type');
add_action('wp_ajax_nopriv_set_delivery_type', 'set_delivery_type');
function set_delivery_type()
{
    WC()->session->set("delivery_type", sanitize_text_field($_POST['delivery_type']));
    WC()->shipping->calculate_shipping(WC()->shipping->packages);
    WC()->shipping()->calculate_shipping();
    WC()->cart->calculate_shipping();
    WC()->cart->calculate_totals();
    wp_send_json(['success' => true, 'cart' => WC()->cart]);
    wp_die();
}

// work in progress replacement for ajax-pick-up-point.php
add_action('wp_ajax_set_pick_up_point', 'set_pick_up_point');
add_action('wp_ajax_nopriv_set_pick_up_point', 'set_pick_up_point');
function set_pick_up_point()
{
    if (!empty($_POST['pargoshipping'])) {
        $cleaned = sanitize_post($_POST);
        WC()->session->set('pargo_shipping_address', $cleaned['pargoshipping']);
    }
    wp_send_json(['success' => true]);
    wp_die();
}

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    2.4.4
 */
function run_pargo()
{
    $plugin = new Wp_Pargo();
    $plugin->run();
}

run_pargo();
